import os
import pathlib
from unittest.mock import mock_open, patch

from casa_envoy.auditor import get_value_for
from casa_envoy.palaver import (
    CalibrationLauncher,
    check_calibratable,
    substitute_ppr_paths_for_condor,
)


def test_substitute_ppr_paths_for_condor():
    # cd into test directory in order to use relative paths
    os.chdir("test")

    ppr = "PPR.xml"
    assert os.path.exists(ppr)
    ppr_copy = "PPR-original.xml"
    assert not os.path.exists(ppr_copy)
    original_root_dir = get_value_for(ppr, "RootDirectory")
    original_rel_path = get_value_for(ppr, "RelativePath")
    cwd = pathlib.Path(os.getcwd())
    this_dir = str(cwd.stem)
    up_one_dir = str(cwd.parent)
    mock_settings = {"rootDirectory": "/tmp/workspaces_tmp"}

    with substitute_ppr_paths_for_condor("PPR.xml", mock_settings):
        assert os.path.exists(ppr_copy)
        assert get_value_for(ppr_copy, "RootDirectory") == original_root_dir
        assert get_value_for(ppr_copy, "RelativePath") == original_rel_path
        assert get_value_for(ppr, "RootDirectory") == up_one_dir
        assert get_value_for(ppr, "RelativePath") == this_dir
        assert mock_settings["rootDirectory"] == up_one_dir

    assert not os.path.exists(ppr_copy)
    assert get_value_for(ppr, "RootDirectory") == original_root_dir
    assert get_value_for(ppr, "RelativePath") == original_rel_path
    assert mock_settings["rootDirectory"] == original_root_dir

    # cd back where we came from
    os.chdir("..")


class TestCalibrationLauncher:
    @patch("os.chdir")
    @patch("shutil.copy")
    def test_run(self, mock_copy, mock_os):
        with patch("subprocess.Popen") as sp:
            CalibrationLauncher("test/PPR.xml", "test/test.json").run()
            sp.call_count == 1

    def test_setup_environment(self):
        parameters = {
            "useCasa": False,
            "homeForReprocessing": "/home/casa/packages/pipeline/current",
            "rootDirectory": "/tmp/workspaces_tmp/",
        }
        CalibrationLauncher("PPR.xml", "test.json").setup_environment(parameters=parameters)
        assert os.environ.get("SCIPIPE_ROOTDIR") == parameters["rootDirectory"]
        assert os.environ.get("CASA_HOME") == parameters["homeForReprocessing"]
        assert os.environ.get("PPR_FILENAME") == "PPR.xml"

    @patch("os.chdir")
    @patch("os.getcwd")
    def test_check_logs(self, mock_os_cwd, mock_os_dir):
        with patch("builtins.open", mock_open()) as o:
            CalibrationLauncher("PPR.xml", "test.json").check_logs(parent_path=".")
            o.call_count == 1


def test_check_calibratable():
    check = check_calibratable("test/test.json")
    assert check == True
